@@ -175,6 +175,14 @@ out:
   return display;
 }
 
+static void
+contact_capabilities_changed (TpContact *contact,
+    GParamSpec *pspec,
+    EmpathyContactChooser *self)
+{
+  empathy_individual_view_refilter (self->priv->view);
+}
+
 static void
 get_contacts_cb (TpConnection *connection,
     guint n_contacts,
@@ -191,7 +199,7 @@ get_contacts_cb (TpConnection *connection,
   TpAccount *account;
   TpfPersonaStore *store;
   FolksIndividual *individual;
-  TpfPersona *persona_new;
+  TpfPersona *persona;
   GeeSet *personas;
 
   if (self->priv->add_temp_ctx != ctx)
@@ -201,24 +209,29 @@ get_contacts_cb (TpConnection *connection,
   if (n_contacts != 1)
     return;
 
-  account = g_object_get_data (G_OBJECT (connection), "account");
+  account = tp_connection_get_account (connection);
 
   store = tpf_persona_store_new (account);
   personas = GEE_SET (
       gee_hash_set_new (FOLKS_TYPE_PERSONA, g_object_ref, g_object_unref,
       g_direct_hash, g_direct_equal));
-  persona_new = tpf_persona_new (contacts[0], store);
-  gee_collection_add (GEE_COLLECTION (personas),
-      tpf_persona_new (contacts[0], store));
+
+  persona = tpf_persona_new (contacts[0], store);
+
+  gee_collection_add (GEE_COLLECTION (personas), persona);
 
   individual = folks_individual_new (personas);
 
+  /* listen for updates to the capabilities */
+  tp_g_signal_connect_object (contacts[0], "notify::capabilities",
+      G_CALLBACK (contact_capabilities_changed), self, 0);
+
   /* Pass ownership to the list */
   ctx->individuals = g_list_prepend (ctx->individuals, individual);
 
   individual_store_add_individual_and_connect (self->priv->store, individual);
 
-  g_clear_object (&persona_new);
+  g_clear_object (&persona);
   g_clear_object (&personas);
   g_object_unref (store);
 }
@@ -252,10 +265,6 @@ add_temporary_individuals (EmpathyContactChooser *self,
       if (conn == NULL)
         continue;
 
-      /* One day we'll have tp_connection_get_account()... */
-      g_object_set_data_full (G_OBJECT (conn), "account",
-          g_object_ref (account), g_object_unref);
-
       tp_connection_get_contacts_by_id (conn, 1, &id, G_N_ELEMENTS (features),
           features, get_contacts_cb, self->priv->add_temp_ctx, NULL,
           G_OBJECT (self));
