@@ -782,7 +782,7 @@ theme_adium_append_message (EmpathyChatView *view,
 	EmpathyContact        *sender;
 	TpMessage             *tp_msg;
 	TpAccount             *account;
-	gchar                 *body_escaped;
+	gchar                 *body_escaped, *name_escaped;
 	const gchar           *name;
 	const gchar           *contact_id;
 	EmpathyAvatar         *avatar;
@@ -816,6 +816,8 @@ theme_adium_append_message (EmpathyChatView *view,
 	contact_id = empathy_contact_get_id (sender);
 	action = (empathy_message_get_tptype (msg) == TP_CHANNEL_TEXT_MESSAGE_TYPE_ACTION);
 
+	name_escaped = g_markup_escape_text (name, -1);
+
 	/* If this is a /me probably */
 	if (action) {
 		gchar *str;
@@ -823,7 +825,7 @@ theme_adium_append_message (EmpathyChatView *view,
 		if (priv->data->version >= 4 || !priv->data->custom_template) {
 			str = g_strdup_printf ("<span class='actionMessageUserName'>%s</span>"
 					       "<span class='actionMessageBody'>%s</span>",
-					       name, body_escaped);
+					       name_escaped, body_escaped);
 		} else {
 			str = g_strdup_printf ("*%s*", body_escaped);
 		}
@@ -947,8 +949,10 @@ theme_adium_append_message (EmpathyChatView *view,
 		}
 	}
 
+	name_escaped = g_markup_escape_text (name, -1);
+
 	theme_adium_append_html (theme, func, html, body_escaped,
-				 avatar_filename, name, contact_id,
+				 avatar_filename, name_escaped, contact_id,
 				 service_name, message_classes->str,
 				 timestamp, is_backlog, empathy_contact_is_user (sender));
 
@@ -961,6 +965,7 @@ theme_adium_append_message (EmpathyChatView *view,
 	priv->last_is_backlog = is_backlog;
 
 	g_free (body_escaped);
+	g_free (name_escaped);
 	g_string_free (message_classes, TRUE);
 }
 
@@ -1739,8 +1744,11 @@ adium_info_dup_path_for_variant (GHashTable *info,
 		return g_strdup ("main.css");
 	}
 
-	/* Verify the variant exists, fallback to the first one */
 	variants = empathy_adium_info_get_available_variants (info);
+	if (variants->len == 0)
+		return g_strdup ("main.css");
+
+	/* Verify the variant exists, fallback to the first one */
 	for (i = 0; i < variants->len; i++) {
 		if (!tp_strdiff (variant, g_ptr_array_index (variants, i))) {
 			break;
@@ -1915,8 +1923,16 @@ empathy_adium_data_new_with_info (const gchar *path, GHashTable *info)
 	/* template -> empathy's template */
 	data->custom_template = (template_html != NULL);
 	if (template_html == NULL) {
+		GError *error = NULL;
+
 		tmp = empathy_file_lookup ("Template.html", "data");
-		g_file_get_contents (tmp, &template_html, NULL, NULL);
+
+		if (!g_file_get_contents (tmp, &template_html, NULL, &error)) {
+			g_warning ("couldn't load Empathy's default theme "
+				"template: %s", error->message);
+			g_return_val_if_reached (data);
+		}
+
 		g_free (tmp);
 	}
 
