@@ -38,6 +38,13 @@
 #define DEBUG_FLAG EMPATHY_DEBUG_TP | EMPATHY_DEBUG_CHAT
 #include "empathy-debug.h"
 
+typedef struct {
+	gchar          *name;
+	guint           id;
+	TpPropertyFlags flags;
+	GValue         *value;
+} EmpathyTpChatProperty;
+
 struct _EmpathyTpChatPrivate {
 	TpAccount             *account;
 	EmpathyContact        *user;
@@ -49,6 +56,16 @@ struct _EmpathyTpChatPrivate {
 	GQueue                *pending_messages_queue;
 	gboolean               had_properties_list;
 	GPtrArray             *properties;
+
+	/* Subject */
+	gboolean               supports_subject;
+	gboolean               can_set_subject;
+	gchar                 *subject;
+
+	/* Room config (for now, we only track the title and don't support
+	 * setting it) */
+	gchar                 *title;
+
 	gboolean               can_upgrade_to_muc;
 
 	GHashTable            *messages_being_sent;
@@ -64,13 +81,14 @@ enum {
 	PROP_ACCOUNT,
 	PROP_REMOTE_CONTACT,
 	PROP_N_MESSAGES_SENDING,
+	PROP_TITLE,
+	PROP_SUBJECT,
 };
 
 enum {
 	MESSAGE_RECEIVED,
 	SEND_ERROR,
 	CHAT_STATE_CHANGED,
-	PROPERTY_CHANGED,
 	MESSAGE_ACKNOWLEDGED,
 	LAST_SIGNAL
 };
@@ -613,6 +631,10 @@ tp_chat_property_flags_changed_cb (TpProxy         *proxy,
 				property->flags = flags;
 				DEBUG ("property %s flags changed: %d",
 					property->name, property->flags);
+
+				if (!tp_strdiff (property->name, "subject")) {
+					self->priv->can_set_subject = !!(property->flags & TP_PROPERTY_FLAG_WRITE);
+				}
 				break;
 			}
 		}
@@ -626,6 +648,7 @@ tp_chat_properties_changed_cb (TpProxy         *proxy,
 			       GObject         *chat)
 {
 	EmpathyTpChat *self = (EmpathyTpChat *) chat;
+	EmpathyTpChatPrivate *priv = self->priv;
 	guint              i, j;
 
 	if (!self->priv->had_properties_list || !properties) {
@@ -652,8 +675,21 @@ tp_chat_properties_changed_cb (TpProxy         *proxy,
 				}
 
 				DEBUG ("property %s changed", property->name);
-				g_signal_emit (chat, signals[PROPERTY_CHANGED], 0,
-					       property->name, property->value);
+
+				if (!tp_strdiff (property->name, "name") &&
+				    G_VALUE_HOLDS_STRING (property->value)) {
+					g_free (priv->title);
+					priv->title = g_value_dup_string (property->value);
+					g_object_notify (chat, "title");
+				} else if (!tp_strdiff (property->name, "subject") &&
+				           G_VALUE_HOLDS_STRING (property->value)) {
+					priv->supports_subject = TRUE;
+					priv->can_set_subject = !!(property->flags & TP_PROPERTY_FLAG_WRITE);
+					g_free (priv->subject);
+					priv->subject = g_value_dup_string (property->value);
+					g_object_notify (chat, "subject");
+				}
+
 				break;
 			}
 		}
@@ -686,13 +722,13 @@ tp_chat_list_properties_cb (TpProxy         *proxy,
 	GArray            *ids;
 	guint              i;
 
-	self->priv->had_properties_list = TRUE;
-
 	if (error) {
 		DEBUG ("Error listing properties: %s", error->message);
 		return;
 	}
 
+	self->priv->had_properties_list = TRUE;
+
 	ids = g_array_sized_new (FALSE, FALSE, sizeof (guint), properties->len);
 	self->priv->properties = g_ptr_array_sized_new (properties->len);
 	for (i = 0; i < properties->len; i++) {
@@ -722,7 +758,7 @@ tp_chat_list_properties_cb (TpProxy         *proxy,
 	g_array_free (ids, TRUE);
 }
 
-void
+static void
 empathy_tp_chat_set_property (EmpathyTpChat *self,
 			      const gchar   *name,
 			      const GValue  *value)
@@ -774,31 +810,153 @@ empathy_tp_chat_set_property (EmpathyTpChat *self,
 	}
 }
 
-EmpathyTpChatProperty *
-empathy_tp_chat_get_property (EmpathyTpChat *self,
-			      const gchar   *name)
+static void
+update_subject (EmpathyTpChat *self,
+		GHashTable *properties)
 {
-	EmpathyTpChatProperty *property;
-	guint                  i;
+	EmpathyTpChatPrivate *priv = self->priv;
+	gboolean can_set, valid;
+	const gchar *subject;
 
-	if (!self->priv->had_properties_list) {
-		return NULL;
+	can_set = tp_asv_get_boolean (properties, "CanSet", &valid);
+	if (valid) {
+		priv->can_set_subject = can_set;
 	}
 
-	for (i = 0; i < self->priv->properties->len; i++) {
-		property = g_ptr_array_index (self->priv->properties, i);
-		if (!tp_strdiff (property->name, name)) {
-			return property;
-		}
+	subject = tp_asv_get_string (properties, "Subject");
+	if (subject != NULL) {
+		g_free (priv->subject);
+		priv->subject = g_strdup (subject);
+		g_object_notify (G_OBJECT (self), "subject");
 	}
 
-	return NULL;
+	/* TODO: track Actor and Timestamp. */
 }
 
-GPtrArray *
-empathy_tp_chat_get_properties (EmpathyTpChat *self)
+static void
+tp_chat_get_all_subject_cb (TpProxy      *proxy,
+			    GHashTable   *properties,
+			    const GError *error,
+			    gpointer      user_data G_GNUC_UNUSED,
+			    GObject      *chat)
 {
-	return self->priv->properties;
+	EmpathyTpChat *self = EMPATHY_TP_CHAT (chat);
+	EmpathyTpChatPrivate *priv = self->priv;
+
+	if (error) {
+		DEBUG ("Error fetching subject: %s", error->message);
+		return;
+	}
+
+	priv->supports_subject = TRUE;
+	update_subject (self, properties);
+}
+
+static void
+update_title (EmpathyTpChat *self,
+	      GHashTable *properties)
+{
+	EmpathyTpChatPrivate *priv = self->priv;
+	const gchar *title = tp_asv_get_string (properties, "Title");
+
+	if (title != NULL) {
+		if (tp_str_empty (title)) {
+			title = NULL;
+		}
+
+		g_free (priv->title);
+		priv->title = g_strdup (title);
+		g_object_notify (G_OBJECT (self), "title");
+	}
+}
+
+static void
+tp_chat_get_all_room_config_cb (TpProxy      *proxy,
+				GHashTable   *properties,
+				const GError *error,
+				gpointer      user_data G_GNUC_UNUSED,
+				GObject      *chat)
+{
+	EmpathyTpChat *self = EMPATHY_TP_CHAT (chat);
+
+	if (error) {
+		DEBUG ("Error fetching room config: %s", error->message);
+		return;
+	}
+
+	update_title (self, properties);
+}
+
+static void
+tp_chat_dbus_properties_changed_cb (TpProxy *proxy,
+				    const gchar *interface_name,
+				    GHashTable *changed,
+				    const gchar **invalidated,
+				    gpointer user_data,
+				    GObject *chat)
+{
+	EmpathyTpChat *self = EMPATHY_TP_CHAT (chat);
+
+	if (!tp_strdiff (interface_name, TP_IFACE_CHANNEL_INTERFACE_SUBJECT)) {
+		update_subject (self, changed);
+	}
+
+	if (!tp_strdiff (interface_name, TP_IFACE_CHANNEL_INTERFACE_ROOM_CONFIG)) {
+		update_title (self, changed);
+	}
+}
+
+void
+empathy_tp_chat_set_subject (EmpathyTpChat *self,
+			     const gchar   *subject)
+{
+	if (tp_proxy_has_interface_by_id (self,
+					  TP_IFACE_QUARK_CHANNEL_INTERFACE_SUBJECT)) {
+		tp_cli_channel_interface_subject_call_set_subject (TP_CHANNEL (self), -1,
+								   subject,
+								   tp_chat_async_cb,
+								   "while setting subject", NULL,
+								   G_OBJECT (self));
+	} else {
+		GValue value = { 0, };
+
+		g_value_init (&value, G_TYPE_STRING);
+		g_value_set_string (&value, subject);
+		empathy_tp_chat_set_property (self, "subject", &value);
+		g_value_unset (&value);
+	}
+}
+
+const gchar *
+empathy_tp_chat_get_title (EmpathyTpChat *self)
+{
+	EmpathyTpChatPrivate *priv = self->priv;
+
+	return priv->title;
+}
+
+gboolean
+empathy_tp_chat_supports_subject (EmpathyTpChat *self)
+{
+	EmpathyTpChatPrivate *priv = self->priv;
+
+	return priv->supports_subject;
+}
+
+gboolean
+empathy_tp_chat_can_set_subject (EmpathyTpChat *self)
+{
+	EmpathyTpChatPrivate *priv = self->priv;
+
+	return priv->can_set_subject;
+}
+
+const gchar *
+empathy_tp_chat_get_subject (EmpathyTpChat *self)
+{
+	EmpathyTpChatPrivate *priv = self->priv;
+
+	return priv->subject;
 }
 
 static void
@@ -849,6 +1007,9 @@ tp_chat_finalize (GObject *object)
 	g_queue_free (self->priv->pending_messages_queue);
 	g_hash_table_destroy (self->priv->messages_being_sent);
 
+	g_free (self->priv->title);
+	g_free (self->priv->subject);
+
 	G_OBJECT_CLASS (empathy_tp_chat_parent_class)->finalize (object);
 }
 
@@ -1235,6 +1396,14 @@ tp_chat_get_property (GObject    *object,
 		g_value_set_uint (value,
 			g_hash_table_size (self->priv->messages_being_sent));
 		break;
+	case PROP_TITLE:
+		g_value_set_string (value,
+			empathy_tp_chat_get_title (self));
+		break;
+	case PROP_SUBJECT:
+		g_value_set_string (value,
+			empathy_tp_chat_get_subject (self));
+		break;
 	default:
 		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, param_id, pspec);
 		break;
@@ -1324,6 +1493,24 @@ empathy_tp_chat_class_init (EmpathyTpChatClass *klass)
 							    0, G_MAXUINT, 0,
 							    G_PARAM_READABLE));
 
+	g_object_class_install_property (object_class,
+					 PROP_TITLE,
+					 g_param_spec_string ("title",
+							      "Title",
+							      "A human-readable name for the room, if any",
+							      NULL,
+							      G_PARAM_READABLE |
+							      G_PARAM_STATIC_STRINGS));
+
+	g_object_class_install_property (object_class,
+					 PROP_SUBJECT,
+					 g_param_spec_string ("subject",
+							      "Subject",
+							      "The room's current subject, if any",
+							      NULL,
+							      G_PARAM_READABLE |
+							      G_PARAM_STATIC_STRINGS));
+
 	/* Signals */
 	signals[MESSAGE_RECEIVED] =
 		g_signal_new ("message-received-empathy",
@@ -1355,16 +1542,6 @@ empathy_tp_chat_class_init (EmpathyTpChatClass *klass)
 			      G_TYPE_NONE,
 			      2, EMPATHY_TYPE_CONTACT, G_TYPE_UINT);
 
-	signals[PROPERTY_CHANGED] =
-		g_signal_new ("property-changed",
-			      G_TYPE_FROM_CLASS (klass),
-			      G_SIGNAL_RUN_LAST,
-			      0,
-			      NULL, NULL,
-			      _empathy_marshal_VOID__STRING_BOXED,
-			      G_TYPE_NONE,
-			      2, G_TYPE_STRING, G_TYPE_VALUE);
-
 	signals[MESSAGE_ACKNOWLEDGED] =
 		g_signal_new ("message-acknowledged",
 			      G_TYPE_FROM_CLASS (klass),
@@ -1399,7 +1576,9 @@ tp_chat_iface_init (EmpathyContactListIface *iface)
 }
 
 EmpathyTpChat *
-empathy_tp_chat_new (TpAccount *account,
+empathy_tp_chat_new (
+		     TpSimpleClientFactory *factory,
+		     TpAccount *account,
 		     TpConnection *conn,
 		     const gchar *object_path,
 		     const GHashTable *immutable_properties)
@@ -1411,6 +1590,7 @@ empathy_tp_chat_new (TpAccount *account,
 	g_return_val_if_fail (immutable_properties != NULL, NULL);
 
 	return g_object_new (EMPATHY_TYPE_TP_CHAT,
+			     "factory", factory,
 			     "account", account,
 			     "connection", conn,
 			     "dbus-daemon", conn_proxy->dbus_daemon,
@@ -1611,28 +1791,27 @@ empathy_tp_chat_get_feature_ready (void)
 }
 
 static void
-conn_prepared_cb (GObject *source,
-	GAsyncResult *result,
+tp_chat_prepare_ready_async (TpProxy *proxy,
+	const TpProxyFeature *feature,
+	GAsyncReadyCallback callback,
 	gpointer user_data)
 {
-	EmpathyTpChat *self = user_data;
-	TpChannel *channel = (TpChannel *) self;
-	GError *error = NULL;
-	TpHandle handle;
-	TpConnection *connection = (TpConnection *) source;
+	EmpathyTpChat *self = (EmpathyTpChat *) proxy;
+	TpChannel *channel = (TpChannel *) proxy;
+	TpConnection *connection;
+	gboolean listen_for_dbus_properties_changed = FALSE;
 
-	if (!tp_proxy_prepare_finish (source, result, &error)) {
-		g_simple_async_result_set_from_error (self->priv->ready_result, error);
-		g_error_free (error);
-		g_simple_async_result_complete (self->priv->ready_result);
-		tp_clear_object (&self->priv->ready_result);
-		return;
-	}
+	g_assert (self->priv->ready_result == NULL);
+	self->priv->ready_result = g_simple_async_result_new (G_OBJECT (self),
+		callback, user_data, tp_chat_prepare_ready_async);
+
+	connection = tp_channel_borrow_connection (channel);
 
 	if (tp_proxy_has_interface_by_id (self,
 					  TP_IFACE_QUARK_CHANNEL_INTERFACE_GROUP)) {
 		const TpIntSet *members;
 		GArray *handles;
+		TpHandle handle;
 
 		/* Get self contact from the group's self handle */
 		handle = tp_channel_group_get_self_handle (channel);
@@ -1655,6 +1834,7 @@ conn_prepared_cb (GObject *source,
 		TpCapabilities *caps;
 		GPtrArray *classes;
 		guint i;
+		TpHandle handle;
 
 		/* Get the self contact from the connection's self handle */
 		handle = tp_connection_get_self_handle (connection);
@@ -1700,25 +1880,31 @@ conn_prepared_cb (GObject *source,
 									       NULL, NULL,
 									       G_OBJECT (self), NULL);
 	}
-}
 
-static void
-tp_chat_prepare_ready_async (TpProxy *proxy,
-	const TpProxyFeature *feature,
-	GAsyncReadyCallback callback,
-	gpointer user_data)
-{
-	EmpathyTpChat *self = (EmpathyTpChat *) proxy;
-	TpChannel *channel = (TpChannel *) proxy;
-	TpConnection *connection;
-	GQuark conn_features[] = { TP_CONNECTION_FEATURE_CAPABILITIES, 0 };
+	if (tp_proxy_has_interface_by_id (self,
+					  TP_IFACE_QUARK_CHANNEL_INTERFACE_SUBJECT)) {
+		tp_cli_dbus_properties_call_get_all (channel, -1,
+						     TP_IFACE_CHANNEL_INTERFACE_SUBJECT,
+						     tp_chat_get_all_subject_cb,
+						     NULL, NULL,
+						     G_OBJECT (self));
+		listen_for_dbus_properties_changed = TRUE;
+	}
 
-	g_assert (self->priv->ready_result == NULL);
-	self->priv->ready_result = g_simple_async_result_new (G_OBJECT (self),
-		callback, user_data, tp_chat_prepare_ready_async);
+	if (tp_proxy_has_interface_by_id (self,
+					  TP_IFACE_QUARK_CHANNEL_INTERFACE_ROOM_CONFIG)) {
+		tp_cli_dbus_properties_call_get_all (channel, -1,
+						     TP_IFACE_CHANNEL_INTERFACE_ROOM_CONFIG,
+						     tp_chat_get_all_room_config_cb,
+						     NULL, NULL,
+						     G_OBJECT (self));
+		listen_for_dbus_properties_changed = TRUE;
+	}
 
-	connection = tp_channel_borrow_connection (channel);
-
-	tp_proxy_prepare_async (connection, conn_features,
-		conn_prepared_cb, self);
+	if (listen_for_dbus_properties_changed) {
+		tp_cli_dbus_properties_connect_to_properties_changed (channel,
+								      tp_chat_dbus_properties_changed_cb,
+								      NULL, NULL,
+								      G_OBJECT (self), NULL);
+	}
 }
